using System;
using System.Collections.Generic;
using System.Text;
using System.Net.Sockets;
using System.Net;
using System.Net.NetworkInformation;
using System.Threading;

namespace PortCheck
{
    class Program
    {
        static void Main(string[] args)
        {
            if (args.Length != 2)
            {
                Console.Error.WriteLine("Usage: PortCheck address port");
                return;
            }

            try
            {
                String address = args[0];
                int port = Int32.Parse(args[1]);
                int timeout = 0;
                if (args.Length == 3)
                {
                    timeout = Int32.Parse(args[2]);
                }

                TcpClient client = new TcpClient();
                DateTime start = DateTime.Now;
                TimeSpan time;
                IPHostEntry host = null;

                //
                //  Resolve addresses
                //
                try
                {
                    IPAddress ipAddr;

                    if (!IPAddress.TryParse(address, out ipAddr))
                    {
                        start = DateTime.Now;
                        host = Dns.GetHostEntry(address);
                        time = DateTime.Now - start;

                        Console.WriteLine("Resolved {0} ({1} ms)", address, time.TotalMilliseconds);
                        Console.WriteLine("Hostnames:");
                        Console.WriteLine("\t{0}", host.HostName);
                        foreach (String alias in host.Aliases)
                        {
                            Console.WriteLine("\t{0}", alias);
                        }
                        Console.WriteLine("IP addresses:");
                        foreach (IPAddress addr in host.AddressList)
                        {
                            Console.WriteLine("\t{0}", addr);
                        }

                        Console.WriteLine();
                    }

                    //
                    //  Attempt to connect
                    //
                    try
                    {
                        start = DateTime.Now;
                        client.Connect(address, port);
                        time = DateTime.Now - start;

                        Console.Out.WriteLine("CONNECTED TO {0}:{1} SUCCESFULLY ({2} ms)", address, port, time.TotalMilliseconds);
                    }
                    catch (Exception e)
                    {
                        time = DateTime.Now - start;
                        Console.WriteLine("CONNECTION TO {0}:{1} FAILED ({2} ms): {3}\n", address, port, time.TotalMilliseconds, e.Message);

                        //
                        // Try to ping the host
                        //
                        PingReply reply = null;
                        if (host != null && host.AddressList.Length > 0 )
                        {
                            try
                            {
                                IPAddress addr = host.AddressList[0];
                                Ping ping = new Ping();
                                start = DateTime.Now;
                                reply = ping.Send(addr);
                                time = DateTime.Now - start;
                                Console.WriteLine("PING REPLY ({1} ms): {0}\n", reply.Status.ToString(), reply.RoundtripTime);
                            }
                            catch (Exception ex)
                            {
                                // BUGBUG: This most likely indicates that the network/host is completely unreachable!
                                time = DateTime.Now - start;
                                Console.WriteLine("PING FAILED ({0} ms): {1}\n", time.TotalMilliseconds, ex.Message);
                            }
                        }

                        if (reply != null && reply.Status == IPStatus.Success)
                        {
                            Console.WriteLine("The host is online, but connection to port failed.  Most likely causes:\n" +
                                                "  No service active on that port\n" +
                                                "  Blocked by remote firewall\n" +
                                                "  Blocked by local or network firewall\n"
                                                );
                        }
                        else
                        {
                            Console.WriteLine("The host may be unreachable. Most likely causes:\n" +
                                                "  No network connectivity to host\n" +
                                                "  No service active on that port\n" +
                                                "  Blocked by remote firewall\n" +
                                                "  Blocked by local or network firewall\n"
                                                );
                        }
                    }
                    finally
                    {
                        if (client != null)
                        {
                            client.Close();
                            client = null;
                        }
                    }
                }
                catch (Exception e)
                {
                    time = DateTime.Now - start;
                    Console.WriteLine("DNS LOOKUP FAILED ({0} ms)", time.TotalMilliseconds);
                    Console.WriteLine("Most likely causes:\n" +
                                        "  Host hostname / address does not exist (check spelling)\n" +
                                        "  DNS is down or cannot connect to DNS server (try from another machine)\n"
                                        );
                    Console.Error.WriteLine("Error: {0}", e.Message);
                }
            }
            catch (Exception e)
            {
                Console.WriteLine("ERROR: {0}", e.ToString());
            }
        }
    }
}
